import unittest
import json
from timer import Timer

class TestConnect(unittest.TestCase):
    def __init__(self, *args):
        super().__init__(*args)
        pass

    def setUp(self):
        pass

    def test_connect1(self):
        from iim42652_spi import iim42652
        s = iim42652.getIIM42652()
        self.assertTrue(s.connected)
        print(s)
        print(s.spi)
        
    def test_connect2(self):
        from iim42652_spi import iim42652
        s = iim42652.getIIM42652({'speed': int(1e6)})
        self.assertTrue(s.connected)
        print(s)
        print(s.spi)

    def test_speeds1(self):
        from iim42652_spi import iim42652
        logSpeeds = range(8)
        regs = None
        props = None
        for l in logSpeeds:
            speed = 10**l
            s = iim42652.getIIM42652({'speed': int(speed), 'trace': True})
            self.assertTrue(s.connected)
            self.assertEqual(s.spi.spi.max_speed_hz, speed)
            print(s.spi)
            if regs is not None:
                r = s.getRegisters()
                self.assertEqual(regs, r)
                regs = r
            if props is not None:
                r = s.getPropertyValues()
                self.assertEqual(props, r)
                props = r

    def benchmark(self, func, N=10):
        times = [0] * N
        for i in range(N):
            timer = Timer()
            with timer:
                r = func()
            times[i] = timer()
        avg = sum(times)/N
        #print(f'Benchmark {func}: avg {avg} s')
        return avg

    def test_speeds_fifo(self):
        from iim42652_spi import iim42652
        regs = None
        props = None

        logSpeeds = [5, 6, 7]
        Ns = [10, 100, 1000]
        results = []

        for l in logSpeeds:
            speed = 10**l
            s = iim42652.getIIM42652({'speed': int(speed), 'trace': True})
            self.assertTrue(s.connected)
            self.assertEqual(s.spi.spi.max_speed_hz, speed)

            for n in Ns:
                func = lambda: s.spi.getbytes(0x30, n)
                te = self.benchmark(func)
                rate = n / te
                results.append({'N': n, 'spispeed': speed, 'time': te, 'rate': rate})
                print(f'S={speed}, N={n}, transfer rate: {rate*1e-3:.3f} kB/s')
        #print(json.dumps(results, indent=2))
        maxrate = max([v['rate'] for v in results])
        print(f'top transfer rate: {maxrate*1e-6:.3f} MB/s')
